// Copy a screen from memory to the screen, optimized version
BasicUpstart2(Start)
#import "C64.inc"

// This is my attempt at optimizing draw-screen.asm. The first change
// to make is get rid of DST, and modify an opcode. This saves a total
// of 2,000 cycles.
//
// Getting rid of SRC is more difficult. The caller is going to have to
// pass the colors in the X and Y registers and set the _src parameter
// instead of the zero page. But this also saves 2,000 cycles.
//
// I'm not sure what else can be done for now. 4,000 cycles doesn't sound
// like a lot on a modern machine, but on the C64 that's 1/5th of an
// entire frame on PAL and even more on NTSC.
//
// This also highlights the convenience of having a macro invoke the
// subroutine for you. I changed the parameters to the routine, but
// the Start routine didn't have to be changed at all. Of course, the
// instructions generated in Start changed, but the source code did not.

Start: {
   DRAW_SCREEN(test_screen)
!: jsr KERNAL_GETIN
   cmp #$0d
   bne !-
   rts
}


.macro DRAW_SCREEN(screen) {
   ldx screen
   ldy screen+1
   lda #<screen+2
   sta DrawScreen._src
   lda #>screen+2
   sta DrawScreen._src+1
   jsr DrawScreen
}
DrawScreen: {
.const SRC=$fb
   stx VIC_BACKGROUND_COLOR
   sty VIC_BORDER_COLOR
   lda #<SCREEN_DEFAULT         //copy characters
   sta dst
   lda #>SCREEN_DEFAULT
   sta dst+1
   jsr copy
   lda #<COLOR_RAM              //copy color RAM
   sta dst
   lda #>COLOR_RAM
   sta dst+1
   jsr copy
   rts

copy:
   ldx #4                       //4 blocks of 250
!loopx:
   ldy #0
!loopy:
   lda _src:$ffff,y
   sta dst:$ffff,y
   iny
   cpy #250
   bne !loopy-
   lda _src                     //increment _src and dst
   clc
   adc #250
   sta _src
   bcc !+
   inc _src+1
!: lda dst
   clc
   adc #250
   sta dst
   bcc !+
   inc dst+1
!: dex
   bne !loopx-
   rts
}


// Screens drawn with https://petscii.krissz.hu/
test_screen:
   .byte COLOR_BLACK            //background color
   .byte COLOR_BLACK            //border color
   //character codes (1000 bytes)
   .byte  85, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 73
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 20,  5, 19, 20, 32, 19,  3, 18,  5,  5, 14, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte 107, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64,115
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32,  6,  9, 12, 12, 32, 20,  8,  9, 19, 32, 23,  9, 20,  8, 32, 20,  5, 24, 20, 32,  1, 14,  4, 32,  7, 18,  1, 16,  8,  9,  3, 19, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32,  1,  4,  4, 32,  3, 15, 12, 15, 18, 19, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104,104, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 16,  5, 20, 19,  3,  9,  9, 32,  9, 19, 32, 21, 19,  5,  6, 21, 12, 32,  1, 14,  4, 32, 22,  5, 18, 19,  1, 20,  9, 12,  5, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32,  3,  1, 18,  4, 32,  7,  1, 13,  5, 19, 32,  3,  1, 14, 32, 13,  1, 11,  5, 32, 21, 19,  5, 32, 15,  6, 32, 65, 83, 88, 90, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32,233,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32,233,160,160,160,160,149,147,133,160,146,133,150,133,146,147,133,160,148,133,152,148,160,160,160,160,160,160,160,160,160,160, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160,160, 32, 32, 32, 32, 93
   .byte  93, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 93
   .byte  74, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 64, 75
   //color codes (1000 bytes)
   .byte   7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7
   .byte   7, 14, 14, 14, 14, 14, 14, 14, 14,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7
   .byte   7, 14, 14, 14, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14,  5,  5,  5,  5,  1,  5,  5,  5,  5,  1,  5,  5,  5,  5,  1,  5,  5,  5,  5,  1,  5,  5,  5,  1,  5,  5,  5,  5,  5,  5,  5,  5,  1,  1,  1,  7
   .byte   7, 14, 14, 14, 14, 14,  7,  7,  7,  7,  7,  7, 14, 14,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14, 14,  7, 14, 14,  7,  7,  1,  1,  1,  1,  1, 13, 13, 13,  1,  2,  3,  4,  5,  6,  7, 14, 14, 14, 14,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14,  7,  7,  7,  7,  7, 14, 14, 14, 14,  7, 14, 14,  7,  7,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14,  7,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 14, 14, 14, 14,  7
   .byte   7, 14, 14,  7,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 14, 14, 14, 14,  7
   .byte   7, 14,  7,  7,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1,  1, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14,  1,  1, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 14, 14, 14, 14,  7
   .byte   7, 14, 14, 14,  1,  1, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 14, 14, 14,  7,  7
   .byte   7, 14, 14, 14, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 14, 14, 14,  7,  7
   .byte   7, 14, 14, 14, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 14, 14, 14,  7,  7
   .byte   7, 14, 14, 14, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 14, 14, 14,  7,  7
   .byte   7, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7, 14, 14, 14, 14,  7
   .byte   7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7,  7
   
